/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.51 from the
 * contents of TextBuffer.xs. Do not edit this file, edit TextBuffer.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "TextBuffer.xs"
#define PERL_NO_GET_CONTEXT
#include "module.h"
#include "wrapper_buffer_line.h"
#include <irssi/src/fe-text/textbuffer-formats.h>

#line 16 "TextBuffer.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

#line 160 "TextBuffer.c"

XS_EUPXS(XS_Irssi_COLORING_STRIP); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_COLORING_STRIP)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;
#line 16 "TextBuffer.xs"
	RETVAL = COLORING_STRIP;
#line 173 "TextBuffer.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_COLORING_EXPAND); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_COLORING_EXPAND)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;
#line 23 "TextBuffer.xs"
	RETVAL = COLORING_EXPAND;
#line 192 "TextBuffer.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_COLORING_UNEXPAND); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_COLORING_UNEXPAND)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;
#line 30 "TextBuffer.xs"
	RETVAL = COLORING_UNEXPAND;
#line 211 "TextBuffer.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_COLORING_RAW); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_COLORING_RAW)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;
#line 37 "TextBuffer.xs"
	RETVAL = COLORING_RAW;
#line 230 "TextBuffer.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__TextUI__Line_prev); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__TextUI__Line_prev)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "line");
    {
	Irssi__TextUI__Line	line = irssi_ref_buffer_line_wrap(ST(0))
;
	Irssi__TextUI__Line	RETVAL;
#line 49 "TextBuffer.xs"
	RETVAL = perl_wrap_buffer_line(line->buffer, line->line->prev);
#line 250 "TextBuffer.c"
	{
	    SV * RETVALSV;
	    RETVALSV = perl_buffer_line_bless(RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__TextUI__Line_next); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__TextUI__Line_next)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "line");
    {
	Irssi__TextUI__Line	line = irssi_ref_buffer_line_wrap(ST(0))
;
	Irssi__TextUI__Line	RETVAL;
#line 57 "TextBuffer.xs"
	RETVAL = perl_wrap_buffer_line(line->buffer, line->line->next);
#line 274 "TextBuffer.c"
	{
	    SV * RETVALSV;
	    RETVALSV = perl_buffer_line_bless(RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__TextUI__Line_get_text); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__TextUI__Line_get_text)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "line, coloring");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	Irssi__TextUI__Line	line = irssi_ref_buffer_line_wrap(ST(0))
;
	int	coloring = (int)SvIV(ST(1))
;
#line 66 "TextBuffer.xs"
	GString *str;
	SV *result;
#line 302 "TextBuffer.c"
#line 69 "TextBuffer.xs"
	str = g_string_new(NULL);
	textbuffer_line2text(line->buffer, line->line, coloring, str);
	result = new_pv(str->str);
	XPUSHs(sv_2mortal(result));
	g_string_free(str, TRUE);
#line 309 "TextBuffer.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi__TextUI__Line_get_format); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__TextUI__Line_get_format)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "line");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	Irssi__TextUI__Line	line = irssi_ref_buffer_line_wrap(ST(0))
;
#line 79 "TextBuffer.xs"
	HV *hv;
	AV *av;
	LINE_REC *l;
	TEXT_BUFFER_FORMAT_REC *f;
	int i;
#line 333 "TextBuffer.c"
#line 85 "TextBuffer.xs"
	hv = newHV();
	l = line->line;
	if (l->info.format != NULL) {
		f = l->info.format;
		(void) hv_store(hv, "module", 6, new_pv(f->module), 0);
		(void) hv_store(hv, "format", 6, new_pv(f->format), 0);
		(void) hv_store(hv, "server_tag", 10, new_pv(f->server_tag), 0);
		(void) hv_store(hv, "target", 6, new_pv(f->target), 0);
		(void) hv_store(hv, "nick", 4, new_pv(f->nick), 0);
		av = newAV();
		for (i = 0; i < f->nargs; i++) {
			av_push(av, new_pv(f->args[i]));
		}
		(void) hv_store(hv, "args", 4, newRV_noinc((SV *) av), 0);
	} else {
		(void) hv_store(hv, "text", 4, new_pv(l->info.text), 0);
	}
	XPUSHs(sv_2mortal(newRV_noinc((SV *) hv)));
#line 353 "TextBuffer.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi__TextUI__Line_get_meta); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__TextUI__Line_get_meta)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "line");
    {
	Irssi__TextUI__Line	line = irssi_ref_buffer_line_wrap(ST(0))
;
	Irssi__UI__LineInfoMeta	RETVAL;
#line 108 "TextBuffer.xs"
	RETVAL = line->line->info.meta;
#line 372 "TextBuffer.c"
	{
	    SV * RETVALSV;
	    RETVALSV = plain_bless(RETVAL, "Irssi::UI::LineInfoMeta");
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}

#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_Irssi__TextUI__TextBuffer); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_Irssi__TextUI__TextBuffer)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        (void)newXSproto_portable("Irssi::COLORING_STRIP", XS_Irssi_COLORING_STRIP, file, "");
        (void)newXSproto_portable("Irssi::COLORING_EXPAND", XS_Irssi_COLORING_EXPAND, file, "");
        (void)newXSproto_portable("Irssi::COLORING_UNEXPAND", XS_Irssi_COLORING_UNEXPAND, file, "");
        (void)newXSproto_portable("Irssi::COLORING_RAW", XS_Irssi_COLORING_RAW, file, "");
        (void)newXSproto_portable("Irssi::TextUI::Line::prev", XS_Irssi__TextUI__Line_prev, file, "$");
        (void)newXSproto_portable("Irssi::TextUI::Line::next", XS_Irssi__TextUI__Line_next, file, "$");
        (void)newXSproto_portable("Irssi::TextUI::Line::get_text", XS_Irssi__TextUI__Line_get_text, file, "$$");
        (void)newXSproto_portable("Irssi::TextUI::Line::get_format", XS_Irssi__TextUI__Line_get_format, file, "$");
        (void)newXSproto_portable("Irssi::TextUI::Line::get_meta", XS_Irssi__TextUI__Line_get_meta, file, "$");
#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
